//!
//! @file cdc_task.c,v
//!
//! Copyright (c) 2004 Atmel.
//!
//! Please read file license.txt for copyright notice.
//!
//! @brief This file manages the CDC task.
//!
//!
//! @version 1.6 at90usb128-demo-cdc-1_0_3 $Id: cdc_task.c,v 1.6 2006/03/17 13:06:22 rletendu Exp $
//!
//! @todo
//! @bug
//!/

//_____  I N C L U D E S ___________________________________________________

#include "config.h"
#include "conf_usb.h"
#include "cdc_task.h"
#include "lib_mcu\usb\usb_drv.h"
#include "usb_descriptors.h"
#include "modules\usb\device_chap9\usb_standard_request.h"
#include "usb_specific_request.h"
#include "lib_mcu/uart/uart_lib.h"
#include "uart_usb_lib.h"
#include <stdio.h>

#include "lib_mcu/adc/adc_drv.h"
#include "lib_board/stk_525/stk_525.h"
#include "lcd.h"




//_____ M A C R O S ________________________________________________________





//_____ D E F I N I T I O N S ______________________________________________



//_____ D E C L A R A T I O N S ____________________________________________


volatile U8 cpt_sof;
extern U8    rx_counter;
extern U8    tx_counter;
S_line_coding line_coding;

volatile U8 delay;
volatile bool state=0;


void int_to_string(S32 value, char* string) {
  U8 i;
 
  string[11]='\0';                  
  if( value < 0 ) {                  
    string[0] = '-';              
    value = -value;
  }
  else string[0] = ' ';             
 
  for(i=10; i>=1; i--) {
    string[i]=(value % 10) +'0';     
    value /= 10;
  }
}

void print_uart(char* string, U8 frac) {
 
  U8 i;            // Zhler
  U8 flag=0;       // Merker fr fhrende Nullen
 
  // Vorkommastellen ohne fhrende Nullen ausgeben
  for(i=0; i<=5; i++) {
    
	if (string[i]!='0') {
      printf("%c",string[i]);
      flag = 1;
    }
  }

  printf("%c",string[6]); 
  printf(",");                // Komma ausgeben
 
  // Nachkommastellen ausgeben
  for(i=7; i<(frac+7); i++) printf("%c",string[i]); 
  printf("V");
}


//! @brief This function initializes the hardware ressources required for CDC demo.
//!
//!
//! @param none
//!
//! @return none
//!
//!/
void cdc_task_init(void)
{
//   uart_init();
   Leds_init();
   Joy_init();
   Hwb_button_init();
   Usb_enable_sof_interrupt();
   init_adc();
   Port_init_lcd();
   delay=0;
   lcd_init();
   
  

 

#ifdef AVRGCC
 	fdevopen(uart_usb_putchar,uart_usb_getchar); //for printf redirection 
#endif
}





//! @brief Entry point of the uart cdc management
//!
//! This function links the uart and the USB bus.
//!
//! @param none
//!
//! @return none
void cdc_task(void)
{
   S32 adc_val;
   char adc_string[16];  
   

   if(Is_device_enumerated()) //Enumeration processs OK ?
   {
      if(cpt_sof>=NB_MS_BEFORE_FLUSH && tx_counter!=0 )  //Flush buffer in Timeout
      {
   	   cpt_sof=0;
		   uart_usb_flush();
      }
	 
	 if (Is_joy_left()) state=0;
	 if (Is_joy_right()) state=1;
	 adc_val=Get_adc_p7_val();


	 if(cpt_sof++==0) delay++;

     if(delay==10)
	  {     
		printf("\r                                     \r");
            printf("Port7: %d     ",adc_val);
	      if(state) { 
		   Led1_toggle();
               adc_val=((adc_val*100000)/1023);
               adc_val-=50000;
	      }
              else {
	 	     adc_val=((adc_val*50000)/1023);
		     Led0_toggle();	
              }
            int_to_string(adc_val,adc_string);
            print_uart(adc_string,4);
            Lcd_data;
            write_lcd('A');

            delay=0;	
	  }         
   }          
}





//! @brief sof_action
//!
//! This function increments the cpt_sof counter each times
//! the USB Start Of Frame interrupt subroutine is executed (1ms)
//! Usefull to manage time delays
//!
//! @warning Code:?? bytes (function code length)
//!
//! @param none
//!
//! @return none
void sof_action()
{
   cpt_sof++;
}


